'//+--------------------------------------------------------------------------+
'//|                                                                          |
'//|                                PNGanim                                   |
'//|                              Version 1.0                                 |
'//|                                                                          |
'//|              To play PNG animations created with GIFtoPNG,               |
'//|                   and customize the animation speed.                     |
'//|                                                                          |
'//+--------------------------------------------------------------------------+
'//|                                                                          |
'//|                         Author Patrice TERRIER                           |
'//|                           copyright (c) 2016                             |
'//|                                                                          |
'//|                        pterrier@zapsolution.com                          |
'//|                                                                          |
'//|                          www.zapsolution.com                             |
'//|                                                                          |
'//+--------------------------------------------------------------------------+
'//|                  Project started on : 10-16-2016 (MM-DD-YYYY)            |
'//|                        Last revised : 11-08-2016 (MM-DD-YYYY)            |
'//+--------------------------------------------------------------------------+

#COMPILE EXE "PNGanim.exe"

#INCLUDE "gdimage.inc"
#INCLUDE "winlift.inc"
#INCLUDE "PNGanim.inc"
#RESOURCE "PNGanim.pbr"

%CTRL_WIDTH    = 606
%CTRL_HEIGHT   = 512
%CLIENT_WIDTH  = 608
%CLIENT_HEIGHT = 573

%IDC_CTRL    = -1
%IDC_TIMER   = -2
%IDC_CLOSE   = 1
%IDC_PREV    = 2
%IDC_NEXT    = 3
%IDC_SPEED   = 4
%IDC_SAVE    = 5
%IDC_STATIC1 = 6
%IDC_STATIC2 = 7
%IDC_TASKBAR = 8

%ID_IMAGE     = 1
%ID_LED_SPEED = 2

type PROP
    hMain           as dword
    hCtrl           as dword
    hList           as dword
    MinTrackSizeW   as long
    MinTrackSizeH   as long
    WasSpeed        as dword
    UseSpeed        as dword
    ImageTick       as dword
    szfolder as asciiz * %MAX_PATH
end type

global gP as PROP

function EXEpath() as string
    static zTmp as asciiz * %MAX_PATH
    if (len(zTmp) = 0) then
        if (GetModuleFileName(byval %NULL, zTmp, sizeof(zTmp))) then
            local pBits as byte ptr
            local K, nLen as long
            nLen = len(zTmp)
            pBits = varptr(zTmp) + nLen
            for K = 1 to nLen
                if (@pBits[-K] <> 92) then
                    @pBits[-K] = 0
                else
                    exit for
                end if
            next
        end if
    end if
    function = zTmp
end function

sub List_DeleteAll(byval hList as long)
    if (IsWindow(hList)) then SendMessage(hList, %LB_RESETCONTENT, 0, 0)
end sub

function List_Add(byval hList as long, zTmp as asciiz) as long
    if (IsWindow(hList)) then
        if (len(zTmp)) then function = SendMessage(hList, %LB_ADDSTRING, 0, varptr(zTmp))
    end if
end function

function List_FindString(byval hList as long, zTmp as asciiz) as long
    local nIndex as long
    nIndex = SendMessage(hList, %LB_FINDSTRING, -1, varptr(zTmp))
    if (nIndex > -1) then incr nIndex: function = nIndex
end function

sub List_SelectPlus(byval hList as long, byval nSelected as long)
    if nSelected > 0 then decr nSelected '// Because ListBox is zero based
    if (IsWindow(hList)) then SendMessage(hList, %LB_SETCURSEL, nSelected, 0)
end sub

function List_Count(byval hList as long) as long
    if (IsWindow(hList)) then function = SendMessage(hList, %LB_GETCOUNT, 0, 0)
end function

function List_GetCursel(byval hList as long) as long
    local nSelected as long
    if (IsWindow(hList)) then
        nSelected = SendMessage(hList, %LB_GETCURSEL, 0, 0)
        if (nSelected > -1) then incr nSelected '// Because ListBox is zero based
    end if
    function = nSelected
end function

function List_GetText(byval hList as long, byval nItem as long) as string
    local sItem as string, nLen as long
    if (nItem > 0) then
        decr nItem
        nLen = SendMessage(hList, %LB_GETTEXTLEN, nItem, 0)
        sItem = space$(nLen)
        SendMessage(hList, %LB_GETTEXT, nItem, byval STRPTR(sItem))
        function = sItem
    end if
end function

sub UseThisImage()
    gP.ImageTick = 0: gP.UseSpeed = 0
    local x, y, w, h, nCount as long
    local hBitmap as dword
    local szFile as asciiz * %MAX_PATH
    szFile = List_GetText(gP.hList, List_GetCursel(gP.hList))
    if (ZI_IsPNGanimation(szFile)) then '// This is a GDImage PNG animation
        local anih as ANIHEADER
        hBitmap = ZI_LoadPNGanimation(szFile, anih)
        nCount = anih.nFrame
        gP.UseSpeed = anih.nSpeed
        w = anih.nWidth: h = anih.nHeight
    else
        hBitmap = ZI_CreateBitmapFromFile(szFile, w, h)
        nCount = w \ h: if ((nCount *  h) <> w) then nCount = 1 else w = h
        if (nCount < 4) then
            gP.UseSpeed = 75
        elseif (nCount < 8) then
            gP.UseSpeed = 50
        elseif (nCount < 12) then
            gP.UseSpeed = 37
        else
            gP.UseSpeed = 30
        end if
        w = h
    end if
    x = (%CTRL_WIDTH - w) \ 2
    y = (%CTRL_HEIGHT - h) \ 2
    ZD_DrawBitmapToCtrl(gP.hCtrl, x, y, hBitmap, ZD_ColorARGB(255, 0), %ID_IMAGE, %ZS_VISIBLE)
    ZD_SetObjectFrameCount(%ID_IMAGE, nCount)       '// Save the frame count
    ZD_SetObjectFrameToUse(%ID_IMAGE, 1, %FALSE)    '// Start with frame 1
    ZD_SetObjectHidden(%ID_IMAGE, %ZS_INACTIVE)     '// Disable it
    ZD_SetObjectZorder(%ID_IMAGE, %ZD_ORDER_BOTTOM) '// Set it to bottom of z-order
    ZD_SetObjectAlpha(%ID_IMAGE, 0, %FALSE)         '// Start invisible

    gP.WasSpeed = gP.UseSpeed
    SendMessage(GetDlgItem(gP.hMain, %IDC_SPEED), %TBM_SETPOS, 1, gP.UseSpeed)

    ZD_SetObjectTextEx(%ID_LED_SPEED, DisplayLongValue(gP.UseSpeed), %FALSE)
    ZD_SetObjectAlpha(%ID_LED_SPEED, 255, %FALSE)
    
    SetWindowText(GetDlgItem(gP.hMain, %IDC_STATIC2), "/" + ltrim$(str$(nCount)))

end sub

sub Animate()
    local nAlpha as long
    nAlpha = ZD_GetObjectAlpha(%ID_LED_SPEED)
    if nAlpha > 0 then ZD_SetObjectAlpha(%ID_LED_SPEED, nAlpha - 1, %FALSE)

    nAlpha = ZD_GetObjectAlpha(%ID_IMAGE)
    if nAlpha < 255 then ZD_SetObjectAlpha(%ID_IMAGE, nAlpha + 1, %FALSE)

    local nTickCount as dword: nTickCount = GetTickCount()
    if gP.ImageTick = 0 then gP.ImageTick = nTickCount + gP.UseSpeed
    if nTickCount > gP.ImageTick then
       gP.ImageTick = 0
       local nFrame as long: nFrame = ZD_GetObjectFrameToUse(%ID_IMAGE) + 1
       if (nFrame > ZD_GetObjectFrameCount(%ID_IMAGE)) then nFrame = 1
       ZD_SetObjectFrameToUse(%ID_IMAGE, nFrame, %TRUE)
       SetWindowText(GetDlgItem(gP.hMain, %IDC_STATIC1), str$(nFrame))
   end if
end sub

sub DetectImage()
    local WashFind, hFind as dword
    local nCount as long
    local fd as WIN32_FIND_DATA
    local zTmp as asciiz * %MAX_PATH

    if (gP.hList = 0) then
        gP.hList = CreateWindowEx(0, "LISTBOX", "", %WS_CHILD or %LBS_SORT, 0, 0, 0, 0, gP.hMain, 0, zInstance(), byval %NULL)
    else
        List_DeleteAll(gP.hList)
    end if

    PathCombine(gP.szfolder, EXEpath(), "images\")
    WashFind = FindFirstFile(gP.szfolder + "ani_*.png", fd)
    hFind = WashFind
    do while hFind > 0
       zTmp = fd.cFileName: CharLower(zTmp)
       if (right$(zTmp, 4) = ".png") then
    		   if (List_Add(gP.hList, gP.szfolder + zTmp) > -1) then nCount += 1
       end if
       hFind = FindNextFile(WashFind, fd)
    loop
    FindClose(WashFind)
    if (nCount > 0) then
        List_SelectPlus(gP.hList, 1)
        UseThisImage()
    end if

end sub

sub CreateStaticText(szUseFont as asciiz, byval x as long, byval y as long, byval w as long, byval h as long, _
                     byval nFontSize as long, byval nID as long, byval nColor as long, szValue as asciiz)
    ZD_DrawTextToCtrlEx(gP.hCtrl, szValue, x, y + 1, w, h, nColor, szUseFont, nFontSize, nID, %ZS_VISIBLE, 0, %StringAlignmentCenter) '%StringAlignmentFar)
    ZD_SetObjectAnchorMode(nID, %ANCHOR_CENTER)
    ZD_SetObjectLocked(nID, %TRUE)
end sub

sub AddChildControls()

    '// Create the GDImage graphic control
    local CtrlW, CtrlH as long
    CtrlW = 606: CtrlH = 512
    gP.hCtrl = CreateWindowEx(0, "ZIMAGECTRL", "", %WS_CHILD OR %WS_VISIBLE, 1, 1, CtrlW, CtrlH, gP.hMain, %IDC_CTRL, zInstance(), byval %NULL)
    if (IsWindow(gP.hCtrl)) then
        local UseBackColor as long
        UseBackColor = ZD_ARGB(32, 0,100,128) '// We use a transparent cyan color for the background.
        ZI_SetProperty(gP.hCtrl, %ZI_GradientTop, UseBackColor)
        ZI_SetProperty(gP.hCtrl, %ZI_GradientBottom, UseBackColor)
        '// Draw as thin border around the border
        'skBorder(gP.hCtrl, 0, 0, 608, 514, 0, 1)
    end if

    local hCtrl as dword
    PathCombine(gP.szfolder, EXEpath(), "resources\")
    'hCtrl = skStaticImage(gP.hMain, gP.szfolder + "taskbar_1.png", -1, 514, 610, 61, %IDC_TASKBAR)
    hCtrl = skStaticImage(gP.hMain, gP.szfolder + "frame.png", -2, 0, 612, 576, %IDC_TASKBAR)

    hCtrl = skButtonImage(gP.hMain, gP.szfolder + "BTN_Close.png", 278, 518, %IDC_CLOSE, 0)
    skCreateToolTip(hCtrl, "Close")

    hCtrl = skButtonImage(gP.hMain, gP.szfolder + "BTN_Left.png", 223, 530, %IDC_PREV, 0)
    skCreateToolTip(hCtrl, "Previous")

    hCtrl = skButtonImage(gP.hMain, gP.szfolder + "BTN_Right.png", 329, 530, %IDC_NEXT, 0)
    skCreateToolTip(hCtrl, "Next")

    hCtrl = CreateWindowEx(0,"MSCTLS_TRACKBAR32", "", %WS_CHILD OR %WS_VISIBLE OR %TBS_HORZ OR %TBS_BOTTOM OR %TBS_AUTOTICKS, _
                           10, 529, 200, 30, gP.hMain, %IDC_SPEED, zInstance(), byval %NULL)
    '// Setup the trackbar parameters.
    SendMessage(hCtrl, %TBM_SETPAGESIZE, 0, 1)
    SendMessage(hCtrl, %TBM_SETLINESIZE, 0, 1)
    SendMessage(hCtrl, %TBM_SETTICFREQ, 0, 1)
    SendMessage(hCtrl, %TBM_SETRANGE, 1, MAK(LONG, 1, 100)) '// Set the animation range
    SendMessage(hCtrl, %TBM_SETPOS, 1, 50)
    skSkinChildCtrl(hCtrl, 0)
    skCreateToolTip(hCtrl, "Animation speed")

    hCtrl = skButtonImage(gP.hMain, gP.szfolder + "button.png", 427, 531, %IDC_SAVE, 0)
    skCreateToolTip(hCtrl, "Become the new animation speed")

    hCtrl = CreateWindowEx(0, "STATIC", "", %WS_CHILD OR %WS_VISIBLE OR %SS_RIGHT, _
                           511, 533, 40, 30, gP.hMain, %IDC_STATIC1, zInstance(), byval %NULL)
    skSkinChildCtrl(hCtrl, 0)
    skSetLabelFont(hCtrl, gP.szfolder + "ft00.ttf", 20, ZD_ARGB(255, 255,255,255), %FontStyleRegular)
    hCtrl = CreateWindowEx(0, "STATIC", "", %WS_CHILD OR %WS_VISIBLE OR %SS_LEFT, _
                           548, 533, 40, 30, gP.hMain, %IDC_STATIC2, zInstance(), byval %NULL)
    skSkinChildCtrl(hCtrl, 0)
    skSetLabelFont(hCtrl, gP.szfolder + "ft00.ttf", 20, ZD_ARGB(255, 255,255,255), %FontStyleRegular)

    '// Use a private font to display animation speed.
    local x, y, w, h, nFontSize, nColor, nValue as long
    nValue = 0
    w = 150: h = 60: x = (CtrlW - w) \ 2 - 4: y = CtrlH - h: nFontSize = 60: nColor = &HFF037BFA '// ZD_ARGB(255, 3,123,250)
    CreateStaticText(gP.szfolder + "lcd.ttf", x, y, w, h, nFontSize, %ID_LED_SPEED, nColor, DisplayLongValue(nValue))
end sub

function DisplayLongValue(byval nValue as long) as string
    local sRet as string
    sRet = str$(abs(nValue))
    sRet = ltrim$(sRet)
    if (nValue < 0) then sRet = "-" + sRet
    function = sRet
end function

function SaveAnimationSpeed() as long
    local anih as ANIHEADER
    local nRet as long
    local szFile as asciiz * %MAX_PATH
    local hFile as dword
    szFile = List_GetText(gP.hList, List_GetCursel(gP.hList))
    hFile = CreateFile(szFile, %GENERIC_READ OR %GENERIC_WRITE, %FILE_SHARE_READ OR %FILE_SHARE_WRITE, byval %NULL, %OPEN_EXISTING, %FILE_ATTRIBUTE_NORMAL, 0)
    if (hFile <> %INVALID_HANDLE_VALUE) then
        local nSize as long, dwBytesRead as dword
        nSize = sizeof(anih)
        if (SetFilePointer(hFile, GetFileSize(hFile, byval %NULL) - nSize, byval %NULL, 0) > 0) then
            if (ReadFile(hFile, byval varptr(anih), nSize, dwBytesRead, byval %NULL)) then
                if (anih.nSign = %ANIM) then
                    local dwByttesWritten as dword
                    anih.nSpeed = gP.UseSpeed
                    if (SetFilePointer(hFile, GetFileSize(hFile, byval %NULL) - nSize, byval %NULL, 0) > 0) then
                        if (WriteFile(hFile, byval varptr(anih), nSize, dwByttesWritten, byval %NULL)) then
                            nRet = -1
                        end if
                    end if
                end if
            end if
        end if
        CloseHandle(hFile)
    end if
    function = nRet
end function

sub PlaySoundBeep()
    PathCombine(gP.szfolder, EXEpath(), "resources\beep.wav")
    PlaySound(gP.szfolder, byval %NULL, %SND_FILENAME or %SND_ASYNC)
end sub

function WndProc(byval hWnd as dword, byval Msg as dword, byval wParam as dword, byval lParam as long) as long
    local nItem, wmID, wmEvent as long
    local lpr as RECT ptr
    local pMM as MINMAXINFO ptr
    local tmi as MONITORINFO, hMonitor as dword

    select case Msg
    case %WM_CREATE
         SetTimer(hWnd, %IDC_TIMER, 0, 0) '// 60 FPS

    case %WM_GETMINMAXINFO
         pMM = lParam
         @pMM.ptMinTrackSize.x = gP.MinTrackSizeW
         @pMM.ptMinTrackSize.y = gP.MinTrackSizeH
         function = 0: exit function

    case %WM_MOVING '// Keep the window into the working area
         hMonitor = MonitorFromWindow(hWnd, %MONITOR_DEFAULTTONEAREST)
         lpr = lParam
         if (@lpr.nTop < 0) then @lpr.nTop = 0
         @lpr.nRight = @lpr.nLeft + gP.MinTrackSizeW
         @lpr.nBottom = @lpr.nTop + gP.MinTrackSizeH
         tmi.cbSize = sizeof(tmi)
         if (GetMonitorInfo(hMonitor, tmi)) then
             if (@lpr.nBottom > tmi.rcWork.nBottom) then
                 @lpr.nBottom = tmi.rcWork.nBottom
                 @lpr.nTop = @lpr.nBottom - gP.MinTrackSizeH
             end if
         end if
         function = %TRUE: exit function

    case %WM_HSCROLL
         if (GetDlgCtrlID(lParam) = %IDC_SPEED) then
            '// Do not use nPosition, because it could be NULL.
            gP.UseSpeed = SendMessage(lParam, %TBM_GetPos, 0, 0)
            'skSetToolTipText(GetDlgItem(hWnd, %IDC_SPEED), ltrim$(str$(gP.UseSpeed)))
            ZD_SetObjectAlpha(%ID_LED_SPEED, 255, %FALSE)
            ZD_SetObjectTextEx(%ID_LED_SPEED, DisplayLongValue(gP.UseSpeed), %TRUE)
         end if

    case %WM_COMMAND
         wmID = lo(INTEGER, wParam)
         wmEvent = hi(INTEGER, wParam)
         select case long (wmID)

         case %IDC_PREV
              nItem = List_GetCursel(gP.hList)
              if (nItem > 1) then
                  List_SelectPlus(gP.hList, nItem - 1)
              else
                  List_SelectPlus(gP.hList, List_Count(gP.hList))
              end if
              UseThisImage()

         case %IDC_NEXT
              nItem = List_GetCursel(gP.hList)
              if (nItem < List_Count(gP.hList)) then
                  List_SelectPlus(gP.hList, nItem + 1)
              else
                  List_SelectPlus(gP.hList, 1)
              end if
              UseThisImage()

         case %IDC_CLOSE
              PostMessage(hWnd, %WM_SYSCOMMAND, %SC_CLOSE, 0)

         case %IDC_SAVE
              ZD_SetObjectAlpha(%ID_LED_SPEED, 255, %FALSE)
              SaveAnimationSpeed()
              PlaySoundBeep()
         end select

    case %WM_TIMER
         Animate()

    case %WM_NCHITTEST
         function = %HTCAPTION: exit function

    case %WM_DESTROY
         KillTimer(hWnd, %IDC_TIMER)
         PostQuitMessage(0)
         function = 0: exit function
    end select

    function = DefWindowProc(hWnd, Msg, wParam, lParam)

end function

function WinMain (byval hInstance as dword, byval hPrevInstance as dword, _
                  byval lpCmdLine as asciiz ptr, byval nCmdShow as long) as long
    local nRet as long
    local Msg as tagMsg
    local wcx as WndClassEx
    local zClass, zTitle as asciiz * 80
    local lpr as RECT
    local dwStyle, dwStyleEx, hMutex, hFound, hCtrl, hDC, hRichEdit as dword
    local x, y, IsInitialized as long

    zClass = "PNGANIM"

    hMutex = CreateMutex(byval %Null, 0, zClass)
    if (hMutex) then
        if (GetLastError() = %ERROR_ALREADY_EXISTS) then
            do
               hFound = FindWindow(zClass, ""): if hFound then exit do
               while PeekMessage(Msg, %NULL, %NULL, %NULL, %PM_REMOVE): wend
            loop
            if (IsIconic(hFound)) then ShowWindow(hFound, %SW_RESTORE)
            SetForeGroundWindow(hFound)
            function = 0
            exit function
        end if
    end if

    wcx.cbSize = sizeof(wcx)
    IsInitialized = GetClassInfoEx(hInstance, zClass, wcx)
    if (IsInitialized     = 0) then
        wcx.style         = %CS_HREDRAW or %CS_VREDRAW
        wcx.lpfnWndProc   = codeptr(WndProc)
        wcx.cbClsExtra    = 0
        wcx.cbWndExtra    = 0
        wcx.hInstance     = hInstance
        wcx.hIcon         = LoadIcon(wcx.hInstance, "PROGRAM")
        wcx.hCursor       = LoadCursor(%NULL, byval %IDC_ARROW)
        wcx.hbrBackground = %NULL
        wcx.lpszMenuName  = %NULL
        wcx.lpszClassName = varptr(zClass)
        wcx.hIconSm       = wcx.hIcon
        if (RegisterClassEx(wcx)) then IsInitialized = %TRUE
    end if

    if (IsInitialized) then

        '// Load the GDImage.dll
        ZI_LoadDLL("")

        '// Window Styles
        dwStyleEx = 0: dwStyle = %WS_POPUP or %WS_CLIPSIBLINGS or %WS_CLIPCHILDREN
        '// Adjust window to the client size
        SetRect(lpr, 0, 0, %CLIENT_WIDTH, %CLIENT_HEIGHT)
        AdjustWindowRectEx(lpr, dwStyle, %FALSE, dwStyleEx)
        x = max&((GetSystemMetrics(%SM_CXSCREEN) - (lpr.nRight - lpr.nLeft)) \ 2, 0)
        y = max&((GetSystemMetrics(%SM_CYSCREEN) - (lpr.nBottom - lpr.nTop)) \ 2, 0)

        '// Create the MAIN window
        gP.hMain = CreateWindowEx(dwStyleEx, zClass, zTitle, dwStyle, x, y, lpr.nRight - lpr.nLeft, lpr.nBottom - lpr.nTop, 0, 0, hInstance, byval %NULL)

        if (gP.hMain) then

            '// WARNING, we use a very specific Skin for this application
            '// it couldn't be used to skin any regular window.
            PathCombine(gP.szfolder, EXEpath(), "resources\")
            if (skInitEngine(gP.szfolder + "skin.sks", "")) then
                skSkinWindow(gP.hMain, "")

                AddChildControls()

                DetectImage()

                GetWindowRect(gP.hMain, lpr)
                gP.MinTrackSizeW = lpr.nRight - lpr.nLeft
                gP.MinTrackSizeH = lpr.nBottom - lpr.nTop

                ShowWindow(gP.hMain, %SW_SHOW)
                SetForegroundWindow(gP.hMain)            ' Slightly Higher Priority
                SetFocus(gP.hMain)                       ' Sets Keyboard Focus to The Window    

                hCtrl = GetDlgItem(gP.hMain, %IDC_SPEED)
                while GetMessage(Msg, %NULL, 0, 0)

                    '// Detect right mouse button click on track bar
                    '// to restore the initial speed value.
                    if (msg.message = %WM_RBUTTONDOWN) then
                        if (msg.hwnd = hCtrl) then 
                            gP.UseSpeed = gP.WasSpeed
                            SendMessage(hCtrl, %TBM_SETPOS, 1, gP.UseSpeed)
                            ZD_SetObjectAlpha(%ID_LED_SPEED, 255, %FALSE)
                            ZD_SetObjectTextEx(%ID_LED_SPEED, DisplayLongValue(gP.UseSpeed), %FALSE)
                        end if
                    end if

                    TranslateMessage(msg)                ' Translate The Message
                    DispatchMessage(msg)                 ' Dispatch The Message
                wend

                nRet = msg.wParam
            end if

        end if

    end if

    if (hMutex) then CloseHandle(hMutex)
    function = nRet
end function